; Lace rel 0.03
; Created by Graechan
; Comments directed to http://gimpchat.com or http://gimpscripts.com
;
; License: GPLv3
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;    GNU General Public License for more details.
;
;    To view a copy of the GNU General Public License
;    visit: http://www.gnu.org/licenses/gpl.html
;
;
; ------------
;| Change Log |
; ------------ 
; Rel 0.01 - Initial Release
; Rel 0.02 - Added Alpha-to-Logo script
; Rel 0.03 - Bugfix Release to set the paint method to paintbrush, and the the dynamics off (in 2.8) during the script
(define (script-fu-lace-alpha image layer
                               lace-color
							   lace-border
							   spacing
							   size
							   keep-selection-in
							   conserve
							   )
							  
    (gimp-image-undo-group-start image)
	
 (let* (
            (width (car (gimp-image-width image)))
			(height (car (gimp-image-height image)))
			(final-width width)
			(final-height height)
			(area (* 1000 1000))
			(alpha (car (gimp-drawable-has-alpha layer)))
		    (sel (car (gimp-selection-is-empty image)))
		    (layer-name (car (gimp-drawable-get-name layer)))
		    (keep-selection keep-selection-in)
			(selection-channel 0)
			(lace-layer 0)
			(name "Lace_Brush")
			(ver 2.8)
        )
	(cond ((not (defined? 'gimp-image-get-item-position)) (set! ver 2.6))) ;define the gimp version
	
	(gimp-context-push)
	(gimp-context-set-paint-method "gimp-paintbrush")
	(if (= ver 2.8) (gimp-context-set-dynamics "Dynamics Off"))
    (gimp-context-set-default-colors)

	;;;;scale image to given area if required	
	(gimp-image-scale image 
		(max 1 (min 262144 (round (* width (sqrt (/ area (* width height)))))))
		(max 1 (min 262144 (round (* height (sqrt (/ area (* width height))))))))
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))
	
	
	(if (= alpha FALSE) (gimp-layer-add-alpha layer))
	
;;;;check that a selection was made if not make one	
	(if (= sel TRUE) (set! keep-selection FALSE))
	(if (= sel TRUE) (gimp-selection-layer-alpha layer))
	
;;;;create selection-channel (gimp-selection-load selection-channel)
    (if (= sel FALSE) (begin
	(set! selection-channel (car (gimp-selection-save image)))	
	(gimp-drawable-set-name selection-channel "selection-channel")
    (gimp-image-set-active-layer image layer)))	

;;;;begin the script
    
    (set! lace-layer (car (gimp-layer-new image width height RGBA-IMAGE "Lace" 100 NORMAL-MODE))) 
	(cond ((= ver 2.8)                            ;insert or add the new layer
	(gimp-image-insert-layer image lace-layer (car (gimp-item-get-parent layer)) (car (gimp-image-get-layer-position image layer)))) ;above
	(else (gimp-image-add-layer image lace-layer (car (gimp-image-get-layer-position image layer))))) ;above	

;;;;create the image lace	
	(gimp-context-set-foreground lace-color)
	(the-lace-brush name spacing size)
	(gimp-context-set-brush name)	
	(gimp-edit-stroke lace-layer)
	(gimp-edit-clear lace-layer)
;;;;create the border lace	
	(if (= lace-border TRUE) (begin
	(cond ((= ver 2.8)
	(gimp-image-select-rectangle 
	                           image 
							   2 ;operation { CHANNEL-OP-ADD (0), CHANNEL-OP-SUBTRACT (1), CHANNEL-OP-REPLACE (2), CHANNEL-OP-INTERSECT (3) }
							   (car (gimp-drawable-offsets layer)) ;x 
							   (cadr (gimp-drawable-offsets layer)) ;y 
							   width 
							   height))
	(else 
	(gimp-rect-select image
                        (car (gimp-drawable-offsets layer))
                        (cadr (gimp-drawable-offsets layer))
                        width
                        height
						CHANNEL-OP-REPLACE ;operation{ CHANNEL-OP-ADD (0), CHANNEL-OP-SUBTRACT (1), CHANNEL-OP-REPLACE (2), CHANNEL-OP-INTERSECT (3) }
						FALSE ;feather
						10)))  ;feather radious
	
	(gimp-context-set-foreground lace-color)
	(the-lace-brush name spacing size)
	(gimp-context-set-brush name)	
	(gimp-edit-stroke lace-layer)
	)
	) ;endif
	
	(if (= sel FALSE) (gimp-selection-load selection-channel))

;;;;finish the script

;;;;Scale Image to it's original size;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (gimp-image-scale-full image final-width final-height 2)
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))
;	(gimp-layer-resize-to-image-size lace-layer)
;	(gimp-image-resize-to-layers image)
	
	(if (= conserve FALSE) (set! layer (car (gimp-image-merge-down image lace-layer EXPAND-AS-NECESSARY))))
    (gimp-drawable-set-name layer layer-name)
    (if (= keep-selection FALSE) (gimp-selection-none image))
	(if (= sel FALSE) (gimp-image-remove-channel image selection-channel))
	(if (and (= conserve FALSE) (= alpha FALSE)) (gimp-layer-flatten layer))	
	
	(gimp-displays-flush)
	(gimp-image-undo-group-end image)
	(gimp-context-pop)
	
	
    



 )
) 



(script-fu-register "script-fu-lace-alpha"        		    
  "Lace..."
  "creates a Lace outline to Image with Transparent backgound,or around a selection, with an optional Lace border"
  "Graechan"
  "Graechan - http://gimpchat.com"
  "June 2013"
  "RGB*"
  SF-IMAGE      "image"      0
  SF-DRAWABLE   "drawable"   0
  SF-COLOR      "Lace Color"    "White"
  SF-TOGGLE     "Add Lace Border"   TRUE
  SF-ADJUSTMENT "Lace Spacing (% of Size)" '(80 .1 1000 .1 1 0 0)
  SF-ADJUSTMENT "Lace Size" '(60.00 0 1000 .01 1 2 0)
  SF-TOGGLE     "Keep selection"          FALSE
  SF-TOGGLE     "Keep the Layers"   FALSE
)

(script-fu-menu-register "script-fu-lace-alpha" "<Image>/Filters/Alpha to Logo")

(define (script-fu-lace-logo 
                                      text
									  letter-spacing
									  line-spacing
                                      color 
                                      font-in 
                                      font-size
									  outline
									  lace-color
									  lace-border
									  spacing
									  size
                                      bkg-type 
                                      pattern
                                      bkg-color
									  gradient
									  conserve)
									  
  (let* (
         (image (car (gimp-image-new 256 256 RGB)))         
         (border (/ font-size 4))
		 (font (if (> (string-length font-in) 0) font-in (car (gimp-context-get-font))))
         (size-layer (car (gimp-text-fontname image -1 0 0 text border TRUE font-size PIXELS font)))
         (final-width (car (gimp-drawable-width size-layer)))
         (final-height (car (gimp-drawable-height size-layer)))
		 (active-gradient (car (gimp-context-get-gradient)))
		 (active-fg (car (gimp-context-get-foreground)))
		 (active-bg (car (gimp-context-get-background)))
         (text-layer 0)
         (width 0)
         (height 0)
         (bkg-layer 0)
		 (ver 2.8)
		 (lace-layer 0)
		 (name "Lace_Brush")         
         )
	(cond ((not (defined? 'gimp-image-get-item-position)) (set! ver 2.6))) ;define the gimp version	 
		 
    (gimp-context-push)
	(gimp-context-set-paint-method "gimp-paintbrush")
	(if (= ver 2.8) (gimp-context-set-dynamics "Dynamics Off"))
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))

;;;;Add the text layer for a temporary larger Image size
    (set! text-layer (car (gimp-text-fontname image -1 0 0 text (round (/ 700 4)) TRUE 700 PIXELS font)))
;;;;adjust text 
	(gimp-text-layer-set-justification text-layer 2)
	(gimp-text-layer-set-letter-spacing text-layer letter-spacing)
	(gimp-text-layer-set-line-spacing text-layer line-spacing)
;;;;set the new width and height	
    (set! width (car (gimp-drawable-width text-layer)))
    (set! height (car (gimp-drawable-height text-layer)))    
    (gimp-image-remove-layer image size-layer)
    (gimp-image-resize-to-layers image)

;;;;set the text clolor    
    (gimp-context-set-foreground color)
	(gimp-selection-layer-alpha text-layer)
	(gimp-edit-fill text-layer FOREGROUND-FILL)
    
 ;;;;begin the script   
	(set! lace-layer (car (gimp-layer-new image width height RGBA-IMAGE "Lace" 100 NORMAL-MODE))) 
	(cond ((= ver 2.8)                            ;insert or add the new layer
	(gimp-image-insert-layer image lace-layer (car (gimp-item-get-parent text-layer)) (car (gimp-image-get-layer-position image text-layer)))) ;above
	(else (gimp-image-add-layer image lace-layer (car (gimp-image-get-layer-position image text-layer))))) ;above
	
	(gimp-context-set-foreground lace-color)
	(the-lace-brush name spacing size)
	(gimp-context-set-brush name)
	(if (= lace-border TRUE) (gimp-selection-invert image))
	(gimp-edit-stroke lace-layer)
	(if (= lace-border TRUE) (gimp-selection-invert image))
	(gimp-edit-clear lace-layer)
	(gimp-selection-none image)
 
;;;;create the background layer    
	(cond ((not (= bkg-type 2))
	(set! bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "Background" 100 NORMAL-MODE)))
    (cond ((= ver 2.8)                            ;insert or add the new layer
	(gimp-image-insert-layer image bkg-layer (car (gimp-item-get-parent text-layer)) (+ (car (gimp-image-get-layer-position image text-layer)) 1))) ;below
	(else (gimp-image-add-layer image bkg-layer (+ (car (gimp-image-get-layer-position image text-layer)) 1))) ;below
	) ;endcond	
	)
	) ;endcond
	(if (= outline TRUE) (gimp-edit-clear text-layer))
	(gimp-context-set-pattern pattern)
	(gimp-context-set-background bkg-color)
	(gimp-context-set-gradient gradient)
	(if (> bkg-type 2) (begin 
	(gimp-context-set-foreground active-fg)
	(gimp-context-set-background active-bg)))
	(if (= bkg-type 4)(gimp-context-set-gradient active-gradient))
	(if (= bkg-type 0) (gimp-drawable-fill bkg-layer PATTERN-FILL))		
    (if (= bkg-type 1) (gimp-drawable-fill bkg-layer BACKGROUND-FILL))	
    (if (> bkg-type 2) 
	(begin
	(gimp-selection-none image)
	(gimp-drawable-fill bkg-layer BACKGROUND-FILL)	
	(gimp-edit-blend bkg-layer CUSTOM-MODE NORMAL-MODE GRADIENT-SHAPEBURST-SPHERICAL 100 0 REPEAT-NONE FALSE FALSE 3 0.2 TRUE 0 0 (/ width 2) (/ height 2))))
              
;;;;resize the text-layer		
    (gimp-image-set-active-layer image text-layer)
	(gimp-layer-resize-to-image-size text-layer)    
	
;;;;Scale Image to it's original size;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (gimp-image-scale-full image final-width final-height 2)
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))   
    
;;;;finish the script	
	(if (= conserve FALSE) (set! text-layer (car (gimp-image-merge-visible-layers image EXPAND-AS-NECESSARY))))
    (gimp-drawable-set-name text-layer text)	
	
	(gimp-brush-delete name)
	(gimp-context-pop)

    (gimp-display-new image)
	
    )
  )
  
(script-fu-register "script-fu-lace-logo"
  "Lace..."
  "Create text with a lace outline and an optional Lace border"
  "Graechan"
  "Graechan - http://gimpchat.com"
  "June 2013"
  ""
  SF-TEXT       "Text"    "Love"
  SF-ADJUSTMENT "Letter Spacing" '(0 -100 100 1 5 0 0)
  SF-ADJUSTMENT "Line Spacing" '(0 -100 100 1 5 0 0)
  SF-COLOR      "Text color"         '(0 0 0)
  SF-FONT       "Font"               "ChopinScript"
  SF-ADJUSTMENT "Font size (pixels)" '(200 6 1000 1 1 0 1)
  SF-TOGGLE     "Lace Outline Only"   FALSE
  SF-COLOR      "Lace Color"    "White"
  SF-TOGGLE     "Add Lace Border"   TRUE
  SF-ADJUSTMENT "Lace Spacing (% of Size)" '(80 .1 1000 .1 1 0 0)
  SF-ADJUSTMENT "Lace Size" '(60.00 0 1000 .01 1 2 0)
  SF-OPTION "Background Type" '("Pattern" "Color" "Transparent" "Gradient" "Active Gradient")
  SF-PATTERN    "Pattern"            "Red Cubes"
  SF-COLOR      "Background color"         '(153 153 153)
  SF-GRADIENT   "Background Gradient" "Horizon 2"
  SF-TOGGLE     "Keep the Layers"   FALSE
  )
(script-fu-menu-register "script-fu-lace-logo" "<Image>/File/Create/Logos")
  
(define (the-lace-brush 
                               name
							   spacing
							   size
							   )
							  

 (let* (
            (width 270)
			(height 270)
			(area (* size size))
			(image (car (gimp-image-new width height RGB)))
			(bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "Background" 100 NORMAL-MODE)))
			(ver 2.8)
			(brush-layer 0)
			(brushName    "outlineBrush")
			(path 0)
        )
	(cond ((not (defined? 'gimp-image-get-item-position)) (set! ver 2.6))) ;define the gimp version
	
	(gimp-context-push)
	(gimp-context-set-paint-method "gimp-paintbrush")
	(if (= ver 2.8) (gimp-context-set-dynamics "Dynamics Off"))
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))
	
	(cond ((= ver 2.8)                            ;insert or add the new layer
	(gimp-image-insert-layer image bkg-layer 0 0))
	(else (gimp-image-add-layer image bkg-layer 0)))
	(gimp-drawable-fill bkg-layer BACKGROUND-FILL)

;;;;begin the script
;;;;create the polygon	
	(set! brush-layer (car (gimp-layer-new image width height RGBA-IMAGE "Polygon" 100 NORMAL-MODE))) 
	(cond ((= ver 2.8)                            ;insert or add the new layer
	(gimp-image-insert-layer image brush-layer (car (gimp-item-get-parent bkg-layer)) (car (gimp-image-get-layer-position image bkg-layer)))) ;above
	(else (gimp-image-add-layer image brush-layer (car (gimp-image-get-layer-position image bkg-layer))))) ;above
	
	(the-lace-brush-polygon image
				brush-layer ;layer
				"Polygon" ;name
				(/ 270 2) ;x
				(/ 270 2) ;y
				(/ 270 2) ;radius
				12 ;sides
				0 ;rotation
				FALSE ;strokepath
				"Black" ;scolor
				TRUE
				"Black") ;fcolor
			
			(gimp-image-remove-vectors image (car (gimp-image-get-active-vectors image)))
				
;;;; define new brush for drawing operation
            (gimp-brush-new brushName)
			(gimp-brush-set-shape brushName BRUSH-GENERATED-CIRCLE)    
            (gimp-brush-set-spikes brushName 2)
            (gimp-brush-set-hardness brushName 1.00)                   
            (gimp-brush-set-aspect-ratio brushName 1.0)
            (gimp-brush-set-angle brushName 0.0)                       
            (gimp-brush-set-spacing brushName 194.0)
            (gimp-brush-set-radius brushName 10)            
            (gimp-context-set-brush brushName)
            (gimp-context-set-foreground '(255 255 255))
	
	(cond ((= ver 2.8)
	(gimp-image-select-ellipse image 2 20 20 (- width 40) (- height 40)))
	(else (gimp-ellipse-select image 20 20 (- width 40) (- height 40) 2 TRUE FALSE 0)))
	
	(gimp-edit-stroke brush-layer)
	(gimp-selection-shrink image 20)
	(gimp-brush-delete brushName)
	
;;;; define new brush for drawing operation
            (gimp-brush-new brushName)
			(gimp-brush-set-shape brushName BRUSH-GENERATED-CIRCLE)    
            (gimp-brush-set-spikes brushName 2)
            (gimp-brush-set-hardness brushName 1.00)                   
            (gimp-brush-set-aspect-ratio brushName 1.0)
            (gimp-brush-set-angle brushName 0.0)                       
            (gimp-brush-set-spacing brushName 197.0)
            (gimp-brush-set-radius brushName 3.5)            
            (gimp-context-set-brush brushName)
            (gimp-context-set-foreground '(255 255 255))
				
	(gimp-edit-stroke brush-layer)
	(gimp-brush-delete brushName)
	
;;;;scale image to given area if required	
	(gimp-image-scale image 
	(max 1 (min 262144 (round (* width (sqrt (/ area (* width height)))))))
	(max 1 (min 262144 (round (* height (sqrt (/ area (* width height))))))))
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))

;;;;set the brush path
	(set! path (string-append gimp-directory "/brushes/" (strcat name ".gbr")))
	
    (set! brush-layer (car (gimp-image-flatten image))) ;flatten image for brush
	(gimp-image-convert-grayscale image)                ;convert image to grayscale to save as .gbr brush
	
;;;;Save the brush	
	(file-gbr-save 1 image brush-layer path path spacing name)
	(gimp-brushes-refresh)	

;;;;delete the image
	(gimp-image-delete image)	
	
;;;;finish the script	
	(gimp-displays-flush)

	(gimp-context-pop)
    (gimp-context-set-brush name)
	
 )
)

(define (the-lace-brush-polygon img         ;;extract from Shape Paths Copyright (c) 2004-2008 Jonathan Stipe and Pucelo
				drawable
				name
				x
				y
				radius
				sides
				rotation
				strokepath
				scolor
				fillpath
				fcolor)
  (gimp-image-undo-group-start img)
  (let* ((pathvector (car (gimp-vectors-new img name)))
	 (init (- rotation 90))
	 (deg 0)
	 (i 1)
	 (p (lace-translate-point x y init radius))
	 (strokeid (car (gimp-vectors-bezier-stroke-new-moveto pathvector (car p) (cadr p)))))
    (while (< i sides)
      (set! deg (+ (* (/ i sides) 360) init))
      (set! p (lace-translate-point x y deg radius))
      (gimp-vectors-bezier-stroke-lineto pathvector strokeid (car p) (cadr p))
      (set! i (+ i 1))
    )
    (gimp-vectors-stroke-close pathvector strokeid)
    (gimp-image-add-vectors img pathvector -1)
    (gimp-image-set-active-vectors img pathvector)
    (lace-stroke-fill-path img drawable pathvector strokepath scolor fillpath fcolor)
    (gimp-displays-flush)
  )
  (gimp-image-undo-group-end img))

(define (lace-translate-point x y angle distance)
  (let* ((ang (* angle (/ (* 4 (atan 1.0)) 180)))
	 (nx (+ (* distance (cos ang)) x))
	 (ny (+ (* distance (sin ang)) y)))
    (list nx ny)
  )
)

(define (lace-stroke-fill-path img drawable pathvector strokepath scolor fillpath fcolor)
  (let* ((origfgcolor (car (gimp-context-get-foreground)))
	 (origselection 0)
	 (selOp 0))
    (if (= fillpath TRUE)
      (begin
        (set! origselection (car (gimp-selection-save img)))
        (if (= (car (gimp-selection-is-empty img)) 1)
          (set! selOp 2)
          (set! selOp 3)
        )
        (gimp-context-set-foreground fcolor)
        (gimp-vectors-to-selection pathvector selOp 1 0 0 0)
        (gimp-edit-fill drawable 0)
        (gimp-selection-load origselection)
        (gimp-image-remove-channel img origselection)
      )
    )
    (if (= strokepath TRUE)
      (begin
        (gimp-context-set-foreground scolor)
        (gimp-edit-stroke-vectors drawable pathvector)
        (gimp-context-set-foreground origfgcolor)
      )
    )
    (gimp-context-set-foreground origfgcolor)
  )
)


